==> druid-detector-0.6.tar.gz: druid-detector-0.6/app/__main__.py <==
# vi: set softtabstop=2 ts=2 sw=2 expandtab:
# pylint: disable=W0621

import os.path
import sys
import logging
import argparse
import json
import requests
import mergeconf
from signedapi import signedapi

from . import appname, codename, logger
from .detector import registry
from . import version

# API version, what else is there to say
API_VERSION = 2

config_file_paths = [
  f"{os.path.abspath(os.path.curdir)}/{appname}.conf",
  os.path.expanduser(f"~/.config/{codename}/{appname}.conf"),
  f"/etc/{codename}/{appname}.conf"
]

def determine_config_file():
  """
  Determine configuration file to use.

  Returns:
    Configuration file path or None if nothing suitable found.
  """
  for path in config_file_paths:
    if os.path.exists(path):
      return path
  return None

def main():

  # import detectors.  We do this here so that logging is initialized
  # pylint: disable=import-outside-toplevel,unused-import
  import detectors

  # build base configuration
  config = mergeconf.MergeConf(codename, strict=False)
  config_manager = config.add_section('manager')
  config_manager.add('uri', mandatory=True, description='Base URI for DRUID Manager (not including path)')
  config_manager.add('api_id', mandatory=True, description='API ID received from DRUID administrator')
  config_manager.add('api_key', mandatory=True, description='API key received from DRUID administrator')

  # have each registered detector define its configuration
  for detector in registry:
    logger.debug("About to define configuration for %s", detector.name)
    detector_config = config.add_section(detector.name)
    detector.define_configuration(detector_config)

  # get command-line options
  parser = argparse.ArgumentParser(prog=appname)
  parser.add_argument("-c", "--config", type=str,
                      help="Configuration file",
                      default=determine_config_file())
  parser.add_argument("--dry-run",
                      help="Detect without reporting results",
                      action='store_true')
  parser.add_argument("-d", "--debug",
                      help="Debug output",
                      action='store_true')
  parser.add_argument("-q", "--quiet",
                      help="No output",
                      action='store_true')
  parser.add_argument("--sample-config",
                      help="Generate a sample config file to the console",
                      action='store_true')
  # patch in argument parser
  config.config_argparser(parser)
  args = parser.parse_args()

  if args.sample_config:
    print(config.sample_config())
    sys.exit(0)

  if args.debug:
    logger.setLevel(logging.DEBUG)
  logger.debug("Starting application")

  # determine config file if none specified
  if not args.config:
    logger.fatal(
      "Could not find appropriate configuration file.  Looked for %s",
      ', '.join(config_file_paths)
    )
    sys.exit(1)

  # now read in configuration
  try:
    config.merge_file(args.config)
    config.merge_environment()
    config.merge_args(args)
    config.validate()
  except mergeconf.exceptions.UndefinedSection as e:
    logger.fatal("Unrecognized configuration section: %s", e.section)
    sys.exit(1)
  except mergeconf.exceptions.UndefinedConfiguration as e:
    logger.fatal("Unrecognized configuration item: %s (section %s)", e.item, e.section)
    sys.exit(1)
  except mergeconf.exceptions.MissingConfigurationFile as e:
    logger.fatal("Missing configuration file: %s", e.file)
    sys.exit(1)
  except mergeconf.exceptions.MissingConfiguration as e:
    # TODO: this would be better if e.missing was an array or a hierarchy
    unfiltered = e.missing.split(', ')
    missing = []

    # build list of detectors missing configs
    by_detector = {}
    for item in unfiltered:
      detector = item.split('.')[0]
      if detector == 'manager':
        # not actually a detector, so this is important
        missing.append(item)
      elif detector not in by_detector:
        by_detector[detector] = [item]
      else:
        by_detector[detector].append(item)

    for detector, items in by_detector.items():
      if config[detector].enable:
        missing.extend(items)
    if missing:
      logger.fatal("Missing mandatory configuration items: %s", ', '.join(missing))
      sys.exit(1)
  logger.debug("Merged configuration")

  # create SignedApi
  api = signedapi.SignedApi(codename.upper(), config.manager.uri,
    config.manager.api_id, config.manager.api_key)
  if not api:
    logger.fatal("Could not initialize SignedApi object")
    sys.exit(1)

  # when initialization complete
  logger.info("Started application (%s)", version.version)

  # initialize report, empty except for version number
  report = { 'version': API_VERSION }

  # build report from configured detectors
  for detector in registry:

    name = detector.name

    if name not in config.sections or not config[name].enable:
      # no configuration for registered detector; skip
      logger.debug("No configuration found for %s; skipping", name)
      continue

    # detect and append to report
    report[name] = detector.detect()

  # test if this is a dry run
  if args.dry_run:
    logger.info("Dry run: reporting to console instead of to Manager")
    print(report)
  else:
    # report to Manager
    logger.info("Reporting results")
    try:
      res = api.post('/api/cases/', report)
    except requests.ConnectTimeout as e:
      logger.debug("ConnectionTimeout exception in posting: %s", e)
      logger.fatal("Timed out in connecting to Manager.")
    except requests.ConnectionError as e:
      logger.debug("ConnectionError exception in posting: %s", e)
      logger.fatal("Could not connect to Manager.")
    except requests.RequestException as e:
      logger.debug("RequestException in posting: %s", e)
      logger.fatal("Unknown error in contacting Manager.")
    else:
      if res.status_code >= 400:
        err = json.loads(res.text)
        logger.error("Error in posting: %s", err['error'])
        logger.fatal("Error (%d): %s", res.status_code, res.reason)

  # done
  logger.info("Done the run")

if __name__ == '__main__':
  sys.exit(main())

==> druid-detector-0.6.1.tar.gz: druid-detector-0.6.1/app/__main__.py <==
# vi: set softtabstop=2 ts=2 sw=2 expandtab:
# pylint: disable=W0621

import os.path
import sys
import logging
import argparse
import json
import requests
import mergeconf
from signedapi import signedapi

from . import appname, codename, logger, config
from .detector import registry
from . import version

# API version, what else is there to say
API_VERSION = 2

config_file_paths = [
  f"{os.path.abspath(os.path.curdir)}/{appname}.conf",
  os.path.expanduser(f"~/.config/{codename}/{appname}.conf"),
  f"/etc/{codename}/{appname}.conf"
]

def determine_config_file():
  """
  Determine configuration file to use.

  Returns:
    Configuration file path or None if nothing suitable found.
  """
  for path in config_file_paths:
    if os.path.exists(path):
      return path
  return None

def main():

  # import detectors.  We do this here so that logging is initialized
  # pylint: disable=import-outside-toplevel,unused-import
  import detectors

  # build base configuration
  config_manager = config.add_section('manager')
  config_manager.add('uri', mandatory=True, description='Base URI for DRUID Manager (not including path)')
  config_manager.add('api_id', mandatory=True, description='API ID received from DRUID administrator')
  config_manager.add('api_key', mandatory=True, description='API key received from DRUID administrator')

  # have each registered detector define its configuration
  for detector in registry:
    logger.debug("About to define configuration for %s", detector.name)
    detector_config = config.add_section(detector.name)
    detector.define_configuration(detector_config)

  # get command-line options
  parser = argparse.ArgumentParser(prog=appname)
  parser.add_argument("-c", "--config", type=str,
                      help="Configuration file",
                      default=determine_config_file())
  parser.add_argument("--dry-run",
                      help="Detect without reporting results",
                      action='store_true')
  parser.add_argument("-d", "--debug",
                      help="Debug output",
                      action='store_true')
  parser.add_argument("-q", "--quiet",
                      help="No output",
                      action='store_true')
  parser.add_argument("--sample-config",
                      help="Generate a sample config file to the console",
                      action='store_true')
  # patch in argument parser
  config.config_argparser(parser)
  args = parser.parse_args()

  if args.sample_config:
    print(config.sample_config())
    sys.exit(0)

  if args.debug:
    logger.setLevel(logging.DEBUG)
  logger.debug("Starting application")

  # determine config file if none specified
  if not args.config:
    logger.fatal(
      "Could not find appropriate configuration file.  Looked for %s",
      ', '.join(config_file_paths)
    )
    sys.exit(1)

  # now read in configuration
  try:
    config.merge_file(args.config)
    config.merge_environment()
    config.merge_args(args)
    config.validate()
  except mergeconf.exceptions.UndefinedSection as e:
    logger.fatal("Unrecognized configuration section: %s", e.section)
    sys.exit(1)
  except mergeconf.exceptions.UndefinedConfiguration as e:
    logger.fatal("Unrecognized configuration item: %s (section %s)", e.item, e.section)
    sys.exit(1)
  except mergeconf.exceptions.MissingConfigurationFile as e:
    logger.fatal("Missing configuration file: %s", e.file)
    sys.exit(1)
  except mergeconf.exceptions.MissingConfiguration as e:
    # TODO: this would be better if e.missing was an array or a hierarchy
    unfiltered = e.missing.split(', ')
    missing = []

    # build list of detectors missing configs
    by_detector = {}
    for item in unfiltered:
      detector = item.split('.')[0]
      if detector == 'manager':
        # not actually a detector, so this is important
        missing.append(item)
      elif detector not in by_detector:
        by_detector[detector] = [item]
      else:
        by_detector[detector].append(item)

    for detector, items in by_detector.items():
      if config[detector].enable:
        missing.extend(items)
    if missing:
      logger.fatal("Missing mandatory configuration items: %s", ', '.join(missing))
      sys.exit(1)
  logger.debug("Merged configuration")

  # create SignedApi
  api = signedapi.SignedApi(codename.upper(), config.manager.uri,
    config.manager.api_id, config.manager.api_key)
  if not api:
    logger.fatal("Could not initialize SignedApi object")
    sys.exit(1)

  # when initialization complete
  logger.info("Started application (%s)", version.version)

  # initialize report, empty except for version number
  report = { 'version': API_VERSION }

  # build report from configured detectors
  for detector in registry:

    name = detector.name

    if name not in config.sections or not config[name].enable:
      # no configuration for registered detector; skip
      logger.debug("No configuration found for %s; skipping", name)
      continue

    # detect and append to report
    report[name] = detector.detect()

  # test if this is a dry run
  if args.dry_run:
    logger.info("Dry run: reporting to console instead of to Manager")
    print(report)
  else:
    # report to Manager
    logger.info("Reporting results")
    try:
      res = api.post('/api/cases/', report)
    except requests.ConnectTimeout as e:
      logger.debug("ConnectionTimeout exception in posting: %s", e)
      logger.fatal("Timed out in connecting to Manager.")
    except requests.ConnectionError as e:
      logger.debug("ConnectionError exception in posting: %s", e)
      logger.fatal("Could not connect to Manager.")
    except requests.RequestException as e:
      logger.debug("RequestException in posting: %s", e)
      logger.fatal("Unknown error in contacting Manager.")
    else:
      if res.status_code >= 400:
        err = json.loads(res.text)
        logger.error("Error in posting: %s", err['error'])
        logger.fatal("Error (%d): %s", res.status_code, res.reason)

  # done
  logger.info("Done the run")

if __name__ == '__main__':
  sys.exit(main())
